@@ -3,7 +3,7 @@ from .base import finder    as nrpfinder
 from .base import kmerSetDB
 
 
-__version__ = '1.1.22'
+__version__ = '1.6.3'
 
 __authors__ = '''
 Ayaan Hossain <auh57@psu.edu>
@@ -21,7 +21,7 @@ Version: {}
 Authors: {}
          {}
 
-NRP Calculator offers two modes of operation:
+The Non-Repetitive Parts Calculator offers two modes of operation:
 
 - Finder Mode: Discover toolboxes of non-repetitive parts
                from a list of candidate parts
@@ -30,14 +30,15 @@ NRP Calculator offers two modes of operation:
                based on sequence, structure and model
                constraints
 
-Additionally, a 'background' object is available which can
-be used to store background sequences against which parts
-discovered or designed are ensured to be non-repetitive.
+Additionally, a 'background' object is available that stores
+background sequences. When the 'background' object is used, 
+designed genetic parts will also be non-repetitive with respect
+to these sequences.
 
 You can learn more about the two modes and background via
-  print nrpcalc.background.__doc__
-  print nrpcalc.finder.__doc__
-  print nrpcalc.maker.__doc__
+  print(nrpcalc.background.__doc__)
+  print(nrpcalc.finder.__doc__)
+  print(nrpcalc.maker.__doc__)
 '''.format(
     __version__,
     *__authors__.strip().split('\n'))
@@ -81,7 +82,10 @@ def background(
 
     background / kmerSetDB API Examples
     
+    >>> from pprint import pprint
+    >>>
     >>> import nrpcalc
+    >>>
     >>> my_background_list = [
         'ATGAGATCGTAGCAACC',
         'GACGATTACGTCAGGTA',
@@ -162,17 +166,23 @@ def background(
     >>> len(bkg)
     0
 
-    (9) clear() - removes all k-mers stored in background
+    (9) clear(Lmax=None) - removes all k-mers stored in background,
+                           and optionally resets background Lmax
 
     >>> bkg.add('ATGCTTAGTGCCATACC')
     >>> len(bkg)
     2
     >>> bkg.clear()
-
-    [Background Processing]
-      Removing Seq 1: GGTATGGCAC...
     >>> len(bkg)
     0
+    >>> bkg
+    kmerSetDB stored at ./prj_bkg/ with 0 16-mers
+    >>> bkg.clear(Lmax=17)
+    >>> bkg
+    kmerSetDB stored at ./prj_bkg/ with 0 18-mers
+    >>> bkg.clear(Lmax=15)
+    >>> bkg
+    kmerSetDB stored at ./prj_bkg/ with 0 16-mers
 
     (10) close() - closes background instance; once closed
                    operations on the instance raises error
@@ -184,7 +194,7 @@ def background(
     >>> bkg.add('ATGCTTAGTGCCATACC')
     Traceback (most recent call last):
       File "<stdin>", line 1, in <module>
-      File "nrpcalc/base/kmerSetDB.py", line 96, in wrapper
+      File "nrpcalc/base/kmerSetDB.py", line 97, in wrapper
         raise RuntimeError('kmerSetDB was dropped')
     RuntimeError: kmerSetDB was closed or dropped
 
@@ -204,7 +214,7 @@ def background(
     >>> bkg.add('ATGCTTAGTGCCATACC')
     Traceback (most recent call last):
       File "<stdin>", line 1, in <module>
-      File "nrpcalc/base/kmerSetDB.py", line 96, in wrapper
+      File "nrpcalc/base/kmerSetDB.py", line 97, in wrapper
         raise RuntimeError('kmerSetDB was closed or dropped')
     RuntimeError: kmerSetDB was closed or dropped
     '''
@@ -273,12 +283,13 @@ def finder(
        desc - if True displays progress
               (default=True)
 
-    Returns: A dictionary of DNA strings with integer keys.
+    Returns: A dictionary of DNA or RNA strings with integer keys.
 
     Finder Mode API Example
 
     >>> import nrpcalc
     >>> 
+    >>> # define background corpus
     >>> my_chromosomes = [
         'ATGAGATCGTAGCAACC',
         'GACGATTACGTCAGGTA',
@@ -286,22 +297,25 @@ def finder(
         'CCAGTACGAAAAGGCCC',
         'AAAAAAAAAAAAAAAAA']
     >>> 
+    >>> # initialize background
     >>> genomic_kmers = nrpcalc.background(
         path='./my_genome/',
         Lmax=15)
-    >>> 
     >>> genomic_kmers.multiadd(
         my_chromosomes)
 
     [Background Processing]
       Adding Seq 4: AAAAAAAAAA...
     >>> 
+    >>> # fetch part toolbox
     >>> my_toolbox = [
         'AGAGCTATGACTGACGT',
         'GCAGATAGGGGGTAGTA',
         'TAAAAAAAAAAAAAAAA', # Repeats with last chromosome
-        'CAGATGATGCTAGGACT']
+        'GAGCTATGACTGACGTC'] # Repeats with first part
     >>> 
+    >>> # find non-repetitive subset
+    >>> # with respect to background
     >>> nrpcalc.finder(
         seq_list=my_toolbox,
         Lmax=15,
@@ -316,7 +330,7 @@ def finder(
 
      Check Status: PASS
 
-    [Checking Background]:
+    [Checking Background]
      Background: kmerSetDB stored at ./my_genome/ with 10 16-mers
 
      Check Status: PASS
@@ -327,19 +341,19 @@ def finder(
 
      Check Status: PASS
 
-    Extracted 4 unique sequences out of 4 sequences in 1.693e-05 seconds
+    Extracted 4 unique sequences out of 4 sequences in 3.91e-05 seconds
 
-    Written 4 unique sequences out to ./5ebb5779-5314-41ce-8114-2d858ef41e2e/seq_list.txt in 9.68e-05 seconds
+    Written 4 unique sequences out to ./30c21235-e3f6-47f5-bce1-f99f47053e0b/seq_list.txt in 0.0002167 seconds
 
      [Sequence processing remaining] = 1 
-     [Cliques inserted] = 3
+     [Cliques inserted] = 2 
 
-    Built homology graph in 0.000263 seconds. [Edges = 0] [Nodes = 3]
+    Built homology graph in 0.0006673 seconds. [Edges = 1] [Nodes = 3]
      [Intital Nodes = 4] - [Repetitive Nodes = 1] = [Final Nodes = 3]
 
-     [+] Initial independent set = 0 [0 completex], computing vertex cover on remaining 3 nodes.
+     [+] Initial independent set = 0, computing vertex cover on remaining 0 nodes.
      [+] Vertex Cover Function: NRP 2-approximation
-     [+] Dumping graph into: ./5ebb5779-5314-41ce-8114-2d858ef41e2e/repeat_graph.txt in 0.000402927398682 seconds
+     [+] Dumping graph into: ./30c21235-e3f6-47f5-bce1-f99f47053e0b/repeat_graph.txt in 0.00023055076599121094 seconds
 
     ----------------------
     Now running iteration: 0
@@ -349,19 +363,22 @@ def finder(
       [+] 3 Pendants found
 
      Pendant elimination initiated...
-      [x] Isolated node 0 eliminated
       [x] Isolated node 1 eliminated
-      [x] Isolated node 3 eliminated
+      [x] Pendant node 0 eliminated
+      [+] Node 3 covered
 
 
-     [+] Computed vertex cover of size: 0 (in 0.000123 seconds)
-     [+] Loading graph from: ./5ebb5779-5314-41ce-8114-2d858ef41e2e/repeat_graph.txt
-     [+] Current independent set size:  3
-     [+] Potential nodes for expansion: 0 (projected independent set size: 3)
+     [+] Computed vertex cover of size: 1 (in 0.0002351 seconds)
+     [+] Loading graph from: ./30c21235-e3f6-47f5-bce1-f99f47053e0b/repeat_graph.txt
+     [+] Current independent set size:  2
+     [+] Potential nodes for expansion: 0 (projected independent set size: 2)
      [X] Cannot expand independent set, terminating.
 
-    Non-Repetitive Toolbox Size: 3
-    {0: 'AGAGCTATGACTGACGT', 1: 'GCAGATAGGGGGTAGTA', 3: 'CAGATGATGCTAGGACT'}
+    Non-Repetitive Toolbox Size: 2
+    {1: 'GCAGATAGGGGGTAGTA', 0: 'AGAGCTATGACTGACGT'}
+    >>>
+    >>> genomic_kmers.drop() # we're done with this background
+    True
     '''
     return nrpfinder.nrp_finder(
         seq_list=seq_list,
@@ -370,16 +387,17 @@ def finder(
         allow_internal_repeat=internal_repeats,
         vercov_func=vercov,
         output_file=output_file,
-        verbose=verbose)
+        verbose=verbose,
+        check_constraints=True)
 
 def maker(
     seq_constr,
     struct_constr,
-    target_size,
+    part_type,
     Lmax,
+    target_size,
     internal_repeats=False,
     background=None,
-    part_type='RNA',
     struct_type='mfe',
     seed=None,
     synth_opt=False,
@@ -412,17 +430,28 @@ def maker(
                    followed by either 'G'/'C' for the next
                    four basses, and finally ending with 'T's
     :: struct_constr
-       type - string
+       type - string / None
        desc - a string in dot-parenthesis-x notation that
               describe the secondary base pairing across all
-              nucleotide positions
+              nucleotide positions; a string of only dots implies
+              an absence of any structure constraint
               e.g. '..((xx))..' implies that the first, second,
                    and the last two bases are free to either
-                   base pair or not (dot), the third and fourth
+                   base pair or not ('.'), the third and fourth
                    bases are paired with the eighth and the
-                   seventh bases respectively (parenthesis),
+                   seventh bases respectively ('(' and ')'),
                    while the fifth and the sixth base must
-                   not take part in any base pairing (x) at all
+                   not take part in any base pairing ('x') at all
+    :: part_type
+       type - string
+       desc - must be either 'RNA' or 'DNA' depending on the
+              type of genetic part being designed; ensures
+              that correct folding free-energy parameters are
+              used during structure evaluation
+    :: Lmax
+       type - integer
+       desc - maximum allowed shared repeat length between
+              all sequences in designed toolbox
     :: target_size
        type - integer
        desc - maximum number of genetic parts to be designed
@@ -430,10 +459,6 @@ def maker(
               be reached if the constraints are too strict,
               for example, due to low degeneracy in the given
               sequence constraint, or a low Lmax
-    :: Lmax
-       type - integer
-       desc - maximum allowed shared repeat length between
-              all sequences in designed toolbox
     :: internal_repeats
        type - boolean
        desc - if True then internal repeats in designed parts
@@ -447,13 +472,6 @@ def maker(
        desc - a background object containing k-mers (k=Lmax+1)
               which must be absent in the designed toolbox
               (default=None)
-    :: part_type
-       type - string
-       desc - must be either 'RNA' or 'DNA' depending on the
-              type of genetic part being designed; ensures
-              that correct folding free-energy parameters are
-              used during structure evaluation
-              (default='RNA')
     :: struct_type
        type - string
        desc - must be either 'mfe', 'centroid', or 'both'
@@ -540,7 +558,7 @@ def maker(
        desc - if True displays progress
               (default=True)
 
-    Returns: A dictionary of DNA strings with integer keys.
+    Returns: A dictionary of DNA or RNA strings with integer keys.
     
     Maker Mode API Example
     
@@ -550,22 +568,26 @@ def maker(
     >>>
     >>> # initialize background
     >>> bkg = nrpcalc.background(
-      path='./my_toolbox_kmers',
-      Lmax=Lmax)
+        path='./my_toolbox_kmers',
+        Lmax=Lmax)
     >>> bkg
     kmerSetDB stored at ./my_toolbox_kmers/ with 0 16-mers
     >>>
-    >>> # A local model function
+    >>> # a local model function
     >>> def prevent_cutsites(seq):
-            BamHI = 'GGATCC' # cutsite 1
-            XbaI  = 'TCTAGA' # cutsite 2
-            # evaluate part concurrently
-            if seq[-6:] in [BamHI, XbaI]:
-                return (False, len(seq)-6)
-            else:
-                return (True, None)
+            # part is long enough to evaluate
+            if len(seq) >= 6:
+                BamHI = 'GGATCC' # cutsite 1
+                XbaI  = 'TCTAGA' # cutsite 2
+                # evaluate part concurrently
+                if seq[-6:] in [BamHI, XbaI]:
+                    return (False, len(seq)-6)
+                else:
+                    return (True, None)
+            # part is short enough .. pass
+            return (True, None)
     >>>
-    >>> # A global model function
+    >>> # a global model function
     >>> def optimal_gc_content(seq):
             # compute gc count
             gcount = seq.count('G') * 1.
@@ -583,11 +605,11 @@ def maker(
     >>> promoters_strong = nrpcalc.maker(
         seq_constr='N'*20+'TTGACA'+'N'*17+'TATAAT'+'NNNNNN',
         struct_constr='.'*55,
-        target_size=500,
+        part_type='DNA',
         Lmax=Lmax,
+        target_size=500,
         internal_repeats=False,
         background=None,
-        part_type='DNA',
         local_model_fn=prevent_cutsites,
         global_model_fn=optimal_gc_content)
     WARNING: stacking enthalpies not symmetric
@@ -600,14 +622,14 @@ def maker(
     [Checking Constraints]
       Sequence Constraint: NNNNNNNNNNNNNNNNNNNNTTGACANNNNNNNNNNNNNNNNNTATAATNNNNNN
      Structure Constraint: .......................................................
-        Target Size      : 500 parts
+          Part Type      : DNA
                Lmax      : 15 bp
+        Target Size      : 500 parts
       Internal Repeats   : False
 
      Check Status: PASS
 
     [Checking Arguments]
-       Part Type : DNA
      Struct Type : mfe
       Synth Opt  : False
        Jump Count: 10
@@ -639,14 +661,15 @@ def maker(
     >>>
     >>> # build another toolbox of sigma70 promoters
     >>> # non-repetitive to the previous toolbox
+    >>> # (this is called toolbox chaining)
     >>> promoters_variable = nrpcalc.maker(
         seq_constr='N'*20+'TTGACA'+'N'*16+'WWWWWWW'+'NNNNN',
         struct_constr='.'*55,
-        target_size=500,
+        part_type='DNA',
         Lmax=Lmax,
+        target_size=500,
         internal_repeats=False,
         background=bkg,
-        part_type='DNA',
         local_model_fn=prevent_cutsites,
         global_model_fn=optimal_gc_content)
     WARNING: stacking enthalpies not symmetric
@@ -665,7 +688,7 @@ def maker(
 
      Check Status: PASS
 
-    [Checking Background]:
+    [Checking Background]
      Background: kmerSetDB stored at ./my_toolbox_kmers/ with 20000 16-mers
 
      Check Status: PASS
@@ -701,23 +724,24 @@ def maker(
     >>>
     >>> # verify all promoters are non-repetitive
     >>> assert len(nrpcalc.finder(
-      seq_list=final_toolbox,
-      Lmax=Lmax,
-      verbose=False)) == 1000
+        seq_list=final_toolbox,
+        Lmax=Lmax,
+        verbose=False)) == 1000
     '''
+    # Initialize object
     _maker = nrpmaker.NRPMaker(
         part_type=part_type,
         seed=seed)
     
-    return _maker.nrp_maker(
+    # Compute parts
+    parts = _maker.nrp_maker(
         seq_constr=seq_constr,
-        part_type=part_type,
         struct_constr=struct_constr,
         struct_type=struct_type,
         target_size=target_size,
         homology=Lmax+1,
         allow_internal_repeat=internal_repeats,
-        background=background,        
+        background=background,
         synth_opt=synth_opt,
         local_model_fn=local_model_fn,
         global_model_fn=global_model_fn,
@@ -725,4 +749,9 @@ def maker(
         fail_count=fail_count,
         output_file=output_file,
         verbose=verbose,
-        abortion=True)
\ No newline at end of file
+        abortion=True,
+        check_constraints=True)
+
+    # Cleanups and return
+    del _maker
+    return parts
\ No newline at end of file
